require "Window"
require "GameLib"
require "QuestLib"
require "ApolloTimer"
 
local QuestOverlist = {} 
 
function QuestOverlist:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here

    return o
end

local ktConToColor =
{
	[0] 												= "ffffffff",
	[Unit.CodeEnumLevelDifferentialAttribute.Grey] 		= "ff9aaea3",
	[Unit.CodeEnumLevelDifferentialAttribute.Green] 	= "ff37ff00",
	[Unit.CodeEnumLevelDifferentialAttribute.Cyan] 		= "ff46ffff",
	[Unit.CodeEnumLevelDifferentialAttribute.Blue] 		= "ff309afc",
	[Unit.CodeEnumLevelDifferentialAttribute.White] 	= "ffffffff",
	[Unit.CodeEnumLevelDifferentialAttribute.Yellow] 	= "ffffd400", 
	[Unit.CodeEnumLevelDifferentialAttribute.Orange] 	= "ffff6a00", 
	[Unit.CodeEnumLevelDifferentialAttribute.Red] 		= "ffff0000", 
	[Unit.CodeEnumLevelDifferentialAttribute.Magenta] 	= "fffb00ff", -- Purp
}

function QuestOverlist:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		"QuestLog",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
	
	-- Slash cmds
	Apollo.RegisterSlashCommand("qol", "onChatCommand", self)
	
	-- Events
	Apollo.RegisterEventHandler("ShowQuestLog", "start_open_QuestOverlist_timer", self) -- listen to the original questlog's open event to also open the QuestOverlist mainframe
	Apollo.RegisterEventHandler("QuestFloater", "OnQuestUpdated", self)
	
	-- Timer
	Apollo.RegisterTimerHandler("delay_hook_openQuestLog", "softhook_showQuestLog", self) -- for understanding just ignore this.. this must be done so the timer can be used
	
	-- persist window state (position etc)
	--Apollo.RegisterEventHandler("WindowManagementReady", "OnWindowManagementReady", self)
	
	-- Set up stuff that makes the QOL window move WITH the questlog
	self.init_timer = ApolloTimer.Create(1, false, "PrepareWindowMoveHandling", self) -- time, recurring, handler func, context - mostly self
end

--[[function QuestOverlist:OnWindowManagementReady() -- persists the window state
	Event_FireGenericEvent("WindowManagementRegister", {strName = "QuestOverlist", nSaveVersion=1}) --nSaveVersion is only for Window Management
	Event_FireGenericEvent("WindowManagementAdd", {wnd = self.wndMain, strName = "QuestOverlist", nSaveVersion=1})
end]]--	

function QuestOverlist:start_open_QuestOverlist_timer()
	Apollo.CreateTimer("delay_hook_openQuestLog", 0.5, false) -- the function that shall be called when the timer fires is defined by Apollo.RegisterTimerHandler (in the Init() function)
end

function QuestOverlist:softhook_showQuestLog() -- this function is called when the timer ends / fires ; the timer start when the crabine questlog is shown

	if not self.wndMain:IsShown() then -- prevent unmarking of the selected quest by reloading if already loaded
		self.wndMain:Show(true)

		self:UpdateView()
		
		self:HelperConcatWindows(g_wndProgressLog, self.wndMain)
	end
	
	self.check_ql_visible_timer = ApolloTimer.Create(0.5, true, "TimerPerhapsCloseQolWin", self)
	self.check_ql_visible_timer:Start() 
	
end

-- close qol main win if the carbine questlog was closed
function QuestOverlist:TimerPerhapsCloseQolWin() -- the more accurate way would have been to hook into the close funtion of the original quest log but this solution (timer) is more simple and transparent...
	if not g_wndProgressLog:IsVisible() then
		self.wndMain:Show(false)
		self.check_ql_visible_timer:Stop()
	end
end

function QuestOverlist:HelperConcatWindows(leftWindow, rightWindow)
	-- adjust window
	local qlLeft, qlTop, qlRight, qlBottom
	qlLeft, qlTop, qlRight, qlBottom = leftWindow:GetAnchorOffsets() -- left, top, right, bottom
	
	local qolLeft, qolTop, qolRight, qolBottom
	qolLeft, qolTop, qolRight, qolBottom = rightWindow:GetAnchorOffsets()
	
	local ds = Apollo.GetDisplaySize()
	local verticalOffset = ds["nHeight"]/2
	
	rightWindow:SetAnchorOffsets(qlRight, qlTop+verticalOffset, qlRight + rightWindow:GetWidth(), qlBottom+verticalOffset)
end

function QuestOverlist:OnWindowMove(wndHandler, wndControl)
	self:HelperConcatWindows(g_wndProgressLog, self.qolWin) -- note that this is called in ProgressLog Addon context! Not within QuestOverlist! In PrepareWindowMoveHandling the function is copied to the ProgressLog addon
end

function QuestOverlist:OnLoad()
	self.wndMain = Apollo.LoadForm("QuestOverlist.xml", "MainformNew", nil, self)
	self.wndMain:Show(false)
end

function QuestOverlist:PrepareWindowMoveHandling()
	self.pl = Apollo.GetAddon("ProgressLog")
	if not self.pl or not g_wndProgressLog then -- if the not everything is ready yet:
		self.init_timer = ApolloTimer.Create(1, false, "PrepareWindowMoveHandling", self) -- try to set up again after one more second
	else
		-- make the win only resizeable in horizontal mode, not vertical
		self.wndMain:SetSizingMaximum(800, g_wndProgressLog:GetHeight()-10)
		self.wndMain:SetSizingMinimum(300, g_wndProgressLog:GetHeight()-10)
	
		-- assign move handler
		self.pl.OnWindowMove = self.OnWindowMove -- assign a copy (or ref?) from within QuestOverlist to the object oft the carbine questlog window
		self.pl.HelperConcatWindows = self.HelperConcatWindows -- also assign the helper that does the calc when moving ...
		
		g_wndProgressLog:AddEventHandler("WindowMove", "OnWindowMove") -- dynamically attatch the function we assigned the line before as event handler for window move
		
		-- for the event handler (OnWindowMove) will no longer be executed unter QuestOverlist but under the 
		-- Carbine ProgressLog (which is the parent of QuestLog) we need to give a reference to
		-- our addon's main window in order the move handler can access it
		self.pl.qolWin = self.wndMain 
	end

end

-- update the view as soon as a quest is canceled
function QuestOverlist:OnQuestUpdated(unitTarget,questName,questObj)
	if(self.wndMain:IsShown()) then
		self:UpdateView()
	end
end

--local quests={}
function QuestOverlist:UpdateView()

	--wndMain:FindChild("curquestsbox"):SetText("Quests: " .. QuestLib.GetCount() .. "/40")

	local contentStage = self.wndMain:FindChild("Content")
	contentStage:DestroyChildren()

	local cats = {}
	
	-- group data as needed
	for idx, epiEpisode in pairs(QuestLib.GetTrackedEpisodes()) do -- see line 352 of Carbines QuestTracker.lua (QuestTracker:BuildAll())
		--if (epiEpisode:IsWorldStory() or epiEpisode:IsZoneStory() or epiEpisode:IsRegionalStory() ) then
			-- do nothing
		--else
			for nIdx, queQuest in pairs(epiEpisode:GetAllQuests()) do
				if queQuest:IsInLog() then
					local taskData = {
								title = queQuest:GetTitle(),
								level = queQuest:GetConLevel(),
								task_id = queQuest:GetId(),
								task_obj = queQuest,
								is_task = not ( epiEpisode:IsWorldStory() or epiEpisode:IsZoneStory() or epiEpisode:IsRegionalStory() ), 
								color = ktConToColor[queQuest:GetColoredDifficulty() or 0],
								state = queQuest:GetState() -- completed or not ... 
					}

					local cat = queQuest:GetCategory():GetTitle()
					
					if (cats[cat] == nil) then
						cats[cat] = {}
					end
					table.insert(cats[cat], taskData )
				end
				
			end
		--end
	end
	
	--
	-- stuff for "printing" / visualizing the data
	--	
	local player_level = GameLib.GetPlayerLevel()
	for groupName, tasks in pairs(cats) do
		local catForm = Apollo.LoadForm("QuestOverlist.xml", "CategoryEntry", contentStage, self)
		catForm:SetText(groupName .. " ("..table.getn(tasks)..")")
		for i, taskdata in pairs(tasks) do
			local btn_data = { -- set data to the button for the click handler
				task_id = taskdata["task_id"],
				task_obj = taskdata["task_obj"]
			}
			
			local qEntry = Apollo.LoadForm("QuestOverlist.xml", "QuestEntry", contentStage, self)
			qEntry:SetText("[" .. taskdata["level"] .. "][" .. (taskdata["is_task"] and "T" or "Q") .. "] ".. taskdata["title"] )
			qEntry:SetTextColor(taskdata["color"])
			qEntry:SetData(btn_data)
			if(taskdata["level"] == player_level) then
				-- mark quests on current level
				qEntry:SetSprite("ChatLogSprites:sprChat_BlueBG")
				qEntry:SetText(" "..qEntry:GetText())
			end
			if taskdata["state"] == Quest.QuestState_Achieved or taskdata["state"] == Quest.QuestState_Completed then -- quest completed
				qEntry:SetBGColor("green")
			end
		end
	end

	contentStage:ArrangeChildrenVert()
	
end

function QuestOverlist:onChatCommand(cmd, param)
	self.wndMain:Show(true)

	self:UpdateView() 
end 

function QuestOverlist:OnQuestBtnClick( wndHandler, wndControl, eMouseButton)
	local qObj = wndControl:GetData()["task_obj"]
	if(eMouseButton == 0) then
		Event_FireGenericEvent("ShowQuestLog", qObj) 
		Event_FireGenericEvent("GenericEvent_ShowQuestLog", qObj)
	elseif (eMouseButton == 1) then
		-- right click
		
		-- show the Carbine QuestTrackers context menu and extend it
		-- Note: This is pretty cool and even functional, but won't work if someone has a replacement for the stanadrd quest tracker of carbine
		--[[qti = Apollo.GetAddon("QuestTracker")
		
		qti:ShowQuestRightClick(qObj)

		local buttonPane = qti.wndQuestRightClick:FindChild("RightClickList")
				
		-- add headline 
		local menuEntry = Apollo.LoadForm("QuestOverlist.xml", "CategoryEntry", buttonPane, self)
		menuEntry:SetText("QuestOverlist")
		
		-- adjust height of the context menu
		local dims = {}
		dims[0],dims[1],dims[2],dims[3] = qti.wndQuestRightClick:GetAnchorOffsets()
		qti.wndQuestRightClick:SetAnchorOffsets(dims[0],dims[1],dims[2],dims[3]+200) -- increase context window height

		local extra_buttons = {
			["Set active Quest"] = "onSetActiveQuestButton",
			["Abandon"] = "onAbandonQuest"
		}
		
		for k,v in pairs(extra_buttons) do
			local menuEntry = Apollo.LoadForm("QuestOverlist.xml", "ContextEntry", buttonPane, self)
			menuEntry:SetText(k)
			menuEntry:SetData(v)
		end
		
		buttonPane:ArrangeChildrenVert()
		]]--
		
		-- show own context menu
		self.wndContext = Apollo.LoadForm("QuestOverlist.xml", "ContextMenu", nil, self)
		-- push data to the buttons
		for k,v in pairs(self.wndContext:GetChildren()) do
			v:SetData(qObj)
		end
		
		-- position contextmenu 
		local tCursor = Apollo.GetMouse()
		local nWidth = self.wndContext:GetWidth()
		local knXCursorOffset = 10
		local knYCursorOffset = 25
		self.wndContext:Move(tCursor.x - nWidth + knXCursorOffset, tCursor.y - knYCursorOffset , nWidth, self.wndContext:GetHeight())
	end
end

-- 
-- CONTEXT MENU BUTTON HANDLER
function QuestOverlist:onShowQuestDetails( wndHandler, wndControl, eMouseButton)
	-- open the standard carbine QuestLog
	Event_FireGenericEvent("ShowQuestLog", wndHandler:GetData() ) 
	Event_FireGenericEvent("GenericEvent_ShowQuestLog", wndHandler:GetData() )
	
	self.wndContext:Show(false) -- close/hide context menu
end

function QuestOverlist:onSetActiveQuestButton( wndHandler, wndControl, eMouseButton )
	local q = wndHandler:GetData()
	q:ShowHintArrow()
	GameLib.SetInteractHintArrowObject(q)
	
	self.wndContext:Show(false)
end

function QuestOverlist:onToggleTracked( wndHandler, wndControl, eMouseButton )
	local q = wndHandler:GetData()
	q:ToggleTracked()
	
	self.wndContext:Show(false)
end

function QuestOverlist:onAbandonBtn( wndHandler, wndControl, eMouseButton )
	local q = wndHandler:GetData()
	q:Abandon()
	self:UpdateView()
	
	self.wndContext:Show(false)
end

-----------------------------------------------------------------------------------------------
-- create Instance
-----------------------------------------------------------------------------------------------
local MyInst = QuestOverlist:new()
MyInst:Init()
